import time
import board
from analogio import AnalogIn
from digitalio import DigitalInOut, Direction, Pull
import rotaryio
import neopixel
import usb_hid
from hid_gamepad import Gamepad

PIXEL_PIN = board.NEOPIXEL
ORDER = neopixel.RGB

led = neopixel.NeoPixel(PIXEL_PIN, 1, pixel_order=ORDER)
led[0] = (0, 0, 0)
p0 = AnalogIn(board.A0)
p1 = AnalogIn(board.A1)
p2 = AnalogIn(board.A2)
e0 = rotaryio.IncrementalEncoder(board.D5, board.D6)
e1 = rotaryio.IncrementalEncoder(board.D9, board.D10)
e2 = rotaryio.IncrementalEncoder(board.D11, board.D12)
b0 = DigitalInOut(board.D13)
b0.direction = Direction.INPUT
b0.pull = Pull.UP
led = DigitalInOut(board.D4)
led.direction = Direction.OUTPUT

g = Gamepad(usb_hid.devices)

mode = 0
encoders = [e0, e1, e2]
enmap = [(1, 2), (3, 4), (5, 6)]
encoders_last = [0, 0, 0]
b0_last = False

def range_map(value, in_min, in_max, out_min, out_max):
    return (value - in_min) * (out_max - out_min) // (in_max - in_min) + out_min

def click_button(btn):
    g.press_buttons(btn)
    time.sleep(0.05)
    g.release_buttons(btn)

def encoder_send(id, dir):
    btn = enmap[id][dir]
    if mode:
        btn = btn + 8
    click_button(btn)

while True:
    cv0 = range_map(p0.value, 0, 65535, -127, 127)
    cv1 = range_map(p1.value, 0, 65535, -127, 127)
    cv2 = range_map(p2.value, 0, 65535, -127, 127)
    for idx, en in enumerate(encoders):
        pos = en.position
        if pos != encoders_last[idx]:
            if pos > encoders_last[idx]:
                encoder_send(idx, 0)
            else:
                encoder_send(idx, 1)
            encoders_last[idx] = pos
    if b0.value is False:
        # click_button(7)
        mode = not mode
        led.value = mode
        time.sleep(0.5)

    g.move_joysticks(x=cv0, y=cv1, z=cv2, r_z=None)
    # led[0] = (cv0 + 127, cv1 + 127, cv2 + 127)
    time.sleep(0.05)  # Wait a bit before checking all again
